unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Windows,
  Math,
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, customdrawncontrols,
  customdrawndrawers, customdrawn_common, StdCtrls, ComCtrls;

type

  { TMyButton }

  TMyButton = class(TCustomControl)
  private
     FAngle: Integer;
     FText: string;
     procedure SetAngle(AValue: Integer);
     procedure SetText(AValue: string);
   protected
     procedure Paint; override;
   public
     constructor Create(AOwner: TComponent); override;
   published
     property Angle: Integer read FAngle write SetAngle;
     property Text: string read FText write SetText;
   end;

  { TForm1 }

  TForm1 = class(TForm)
    TrackBar1: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
  private

  public
    MyButton1: TMyButton;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TMyButton }

constructor TMyButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAngle := 0;
  FText := 'Custom Control';
end;

procedure TMyButton.SetAngle(AValue: Integer);
begin
  if FAngle = AValue then
    Exit;
  if ((AValue < 0) or (AValue > 360)) then
    Exit;
  FAngle := AValue;
  Self.Invalidate;
end;

procedure TMyButton.SetText(AValue: string);
begin
  if (FText <> AValue) then
  begin
    FText := AValue;
    Self.Invalidate;
  end;
end;

procedure TMyButton.Paint;
var
  SavedLeft, SavedTop: Integer;
  SavedDC: Integer;
  XForm: TXForm;
  R: TRect;
  Size: TSize;
  lp: TPoint;
  dp: TPoint;
begin
  SavedDC := SaveDC(Self.Canvas.Handle);
  try
    SetGraphicsMode(Self.Canvas.Handle, GM_ADVANCED);
    // Save the current transformation matrix
    GetWorldTransform(Self.Canvas.Handle, XForm);
    XForm.eM11 := Cos(DegToRad(FAngle));
    XForm.eM12 := Sin(DegToRad(FAngle));
    XForm.eM21 := -Sin(DegToRad(FAngle));
    XForm.eM22 := Cos(DegToRad(FAngle));
    XForm.eDx := 0;
    XForm.eDy := 0;
    // Save the original Left and Top properties
    SavedLeft := Self.Left;
    SavedTop := Self.Top;
    // Apply the rotation transformation
    SetWorldTransform(Self.Canvas.Handle, XForm);
    // Get the bounding rectangle of the rotated text
    GetTextExtentPoint32(Self.Canvas.Handle, PChar(FText), Length(FText), Size);
    lp.x := Size.cx;
    lp.y := Size.cy;
    LPtoDP(Self.Canvas.Handle, lp, 1);
    // Calculate the new width and height
    dp.x := Round(lp.x * XForm.eM11 + lp.y * XForm.eM21);
    dp.y := Round(lp.x * XForm.eM12 + lp.y * XForm.eM22);
    // Set the bounding rectangle
    SetRect(R, 0, 0, dp.x, dp.y);
    // Restore the original transformation matrix
    SetWorldTransform(Self.Canvas.Handle, XForm);
    // Draw the background
    Self.Canvas.Brush.Color := clBtnFace; // Set the background color
    Self.Canvas.FillRect(Rect(0, 0, Self.Width, Self.Height)); // Fill the control with the background color
    // Draw the border
    Self.Canvas.Pen.Color := clWindowFrame; // Set the border color
    Self.Canvas.Rectangle(0, 0, Self.Width-1, Self.Height-1); // Draw the border
    // Draw the text
    Self.Canvas.Pen.Color := clBtnText; // Set the font color
    R := Rect(0, 0, Self.Width, Self.Height);
    DrawText(Self.Canvas.Handle, PChar(FText), Length(FText), R, DT_CENTER or DT_VCENTER or DT_SINGLELINE);
    // Restore the original Left and Top properties
    Self.Left := SavedLeft;
    Self.Top := SavedTop;
  finally
    RestoreDC(Self.Canvas.Handle, SavedDC);
  end;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  MyButton1 := TMyButton.Create(Self);
  try
    MyButton1.Width := 150;
    MyButton1.Height := 30;
    MyButton1.Parent := Self;
    MyButton1.Left := 200;
    MyButton1.Top := 200;
    MyButton1.Visible := True;
    MyButton1.Text := 'Test Text';
    MyButton1.Angle := 0;
  finally
  end;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  MyButton1.Angle := Self.TrackBar1.Position;
end;

end.

